package cn.ischoolcode.javabook.dushichat;

/* 
 * 爱校码
 * http://www.ischoolcode.cn/
 *  
 * @author zcj
 */

import java.io.*;

/**
 * 这是一个抽象类，用于文件传输的客户端或服务器。作为
 * DushiServerFileTransfer类与DushiClientFileTransfer类的父类，
 * 这个类包含了公共的数据项和用于读写文件的方法。
 *   
 * @author zcj
 */
public abstract class DushiFileTransfer extends Thread {
	// 这个类管理文件传输

	protected static final int DEFAULT_BLOCKSIZE = 1024;
	protected String fileName = null;
	protected long totalSize = 0;
	protected int totalBlocks = 0;
	protected int blockSize = DEFAULT_BLOCKSIZE;
	protected int expireMins = 0;
	protected long expiryTime = 0;
	protected File tmpFile = null;
	protected RandomAccessFile transFile = null;
	protected int receivedBlocks = 0;
	protected boolean receivedAll = false;

	public DushiFileTransfer(String fn, long ts, int tb, int bs, int minutes) {
		super("都市聊天文件传输线程");

		fileName = fn;
		totalSize = ts;
		totalBlocks = tb;
		blockSize = bs;
		expireMins = minutes;
		expiryTime = (System.currentTimeMillis() + (minutes * 60000));
	}

	public abstract void run();

	/**
	 * 从该文件中读请求块号，并返回数据。 
	 */
	protected byte[] getFileBlock(int blockNumber)
			throws IndexOutOfBoundsException, IOException {
		byte[] data = null;

		if (transFile == null)
			throw (new IOException("未指定传输文件"));
		if (blockNumber >= totalBlocks)
			throw (new IndexOutOfBoundsException("块号 " + blockNumber
					+ " 超出范围"));

		if (blockNumber == (totalBlocks - 1))
			data = new byte[(int) (totalSize % blockSize)];
		else
			data = new byte[blockSize];

		synchronized (transFile) {
			transFile.seek((long) (blockNumber * blockSize));
			transFile.read(data, 0, data.length);
		}

		return (data);
	}

	/**
	 * 写入请求的数据块号到使用提供数据的文件中.
	 */
	protected void putFileBlock(int blockNumber, byte[] data)
			throws IndexOutOfBoundsException, IOException {
		if (transFile == null)
			throw (new IOException("未指定传输文件"));
		if (blockNumber >= totalBlocks)
			throw (new IndexOutOfBoundsException("块号 " + blockNumber
					+ " 超出范围"));

		synchronized (transFile) {
			transFile.seek((long) (blockNumber * blockSize));
			transFile.write(data, 0, data.length);
		}

		receivedBlocks += 1;

		receivedAll = (receivedBlocks >= totalBlocks);
	}
}
